// 	PCF8577 DEMO PROGRAM
// 	2011-03-12 GJO
//
//	V0.01

#include "18F4620.h"
#include <stdlib.h>
#include <stdio.h>

#use delay(clock=16000000, RESTART_WDT)
#fuses HS, NOPROTECT, PUT, BROWNOUT, NOLVP
#use i2c(MASTER, SDA=PIN_C4, SCL=PIN_C3)

void PCF8577_driver(int* p, int offset, int sbr, int n, int sa, int drive, int bank);

#define PCF8577_WR 0x74
#define PCF8577_RD 0x75

#define U1 0
#define U2 1
#define U3 2
#define U4 3

#define U1TO4 0

/*
	PCF8577_driver(int* p, int sbr, int n, int sa, int drive, int bank)

	p		=	address of data array holding the segment data
	offset	=	ioregs data offset
	sbr		=	segment byte reg addr (0-7)
	n		=	number of bytes to load into PCF8577 (1-8)
	sa		=	device subaddress (0-3)
	drive	=	LCD drive type; 0 for direct, 1 for duplex LCD
	bank	=	SBR bank select; 0 = EVEN, 1 = ODD


	PCF8577 Data Sequence:

	START SLAVE ADDRESS - CONTROL WORD - DB1 - DB2 - DB3 - DB4 - STOP

*/

void PCF8577_driver(int* p, int offset, int sbr, int n, int sa, int drive, int bank)
{
	int ctl = 0;
	int* ptr = NULL;

	ptr = &p[offset];

	// Assemble PCF8577 control register bit fields
	ctl |= ((drive & 0x01) << 7) | ((bank & 0x01) << 6) | ((sa & 0x07) << 3) | ((sbr & 0x07));

	i2c_start();
	i2c_write(PCF8577_WR);	// Base PCF8577 Slave Address
    i2c_write(ctl);			// Control word including sub address selector sa

	// write n register bytes to PCF8577
	while(n--)
	{
		i2c_write(*ptr++);
	}
	i2c_stop();
}


/*
	RX LCD SEGMENT MAPPING:
	-----------------------

	REG	IC	DESCR	7		6		5		4		3		2		1		0
	----------------------------------------------------------------------------
	0	U1	NC		x		x		x		x		x		x		x		x
	1		ANNUNC.	FAST	UP		SCAN	DOWN	X		X		X		X
	2		ANNUNC.	x		x		x		x		x		SHARP	NOISE	SLOW
	3		ANNUNC.	SYNC	AM		USB		LSB		FM		WIDE	MED		NAR
	----------------------------------------------------------------------------
	4	U2	100M	x		G		F		E		D		C		B		A
	5		 10M	x		G		F		E		D		C		B		A
	6		  1M	x		G		F		E		D		C		B		A
	7		100K	x		G		F		E		D		C		B		A
	----------------------------------------------------------------------------
	8	U3	 10K	x		G		F		E		D		C		B		A
	9		  1K	x		G		F		E		D		C		B		A
	10		100Hz	x		G		F		E		D		C		B		A
	11		 10Hz	x		G		F		E		D		C		B		A
	----------------------------------------------------------------------------
	12	U4	CH100	x		G		F		E		D		C		B		A
	13		CH10	x		G		F		E		D		C		B		A
	14		CH1		x		G		F		E		D		C		B		A
	15		ANNUNC.	x		X		HI		LOW		PNLLCK	LOCKOUT	VFO		MEM
	----------------------------------------------------------------------------
*/


void main(void)
{

	int LCD_segments[16];

	// Fill array with 0xFF
	memset(LCD_segments, 0xFF, 16);

	// LCD test of all segments
	PCF8577_driver(LCD_segments, 0, 0, 16, U1TO4, 0, 0);	// U1-U4 Auto Increment accross boundaries

	delay_ms(3000);

	// Preset PCF8577 segment registers to demonstrate LCD Receiver readout
	LCD_segments[0] = 0;	// Not connected
	LCD_segments[1] = 32;	// SCAN
	LCD_segments[2] = 1;	// SLOW
	LCD_segments[3] = 8;	// FM
	LCD_segments[4] = 6;	// 1
	LCD_segments[5] = 102;	// 4
	LCD_segments[6] = 102;	// 4
	LCD_segments[7] = 109;	// 5
	LCD_segments[8] = 91;	// 2
	LCD_segments[9] = 109;	// 5
	LCD_segments[10] = 63;	// 0
	LCD_segments[11] = 63;	// 0
	LCD_segments[12] = 6;	// 0
	LCD_segments[13] = 91;	// 0
	LCD_segments[14] = 79;	// 0
	LCD_segments[15] = 2;	// VFO

	// Update all drivers accross boundaries using auto increment
	PCF8577_driver(LCD_segments, 0, 0, 16, U1TO4, 0, 0);

	delay_ms(3000);

	//Write to Display Board individually for each driver IC
	PCF8577_driver(LCD_segments, 0, 0, 4, U1, 0, 0);		// Update only U1
	PCF8577_driver(LCD_segments, 4, 0, 4, U2, 0, 0);		// Update only U2
	PCF8577_driver(LCD_segments, 8, 0, 4, U3, 0, 0);		// Update only U3
	PCF8577_driver(LCD_segments, 12, 0, 4, U4, 0, 0);		// Update only U4

	for(;;);

}